from . import govc


def create(ipath: str):
    govc.run(f"folder.create -json {ipath}", raise_on_failure=True)
    return get(ipath)


def destroy(ipath: str):
    govc.run(f"object.destroy -json {ipath}", raise_on_failure=False)


def get(ipath: str):
    ret = govc.get(f"folder.info -json {ipath}", raise_on_failure=False)
    if not ret:
        return
    items = ret.get("folders")
    if not items:
        return
    ret = items[0]
    ret["mor"] = ret["self"]["value"]
    return ret



"""
Copyright 2023-2023 VMware Inc.
SPDX-License-Identifier: Apache-2.0

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import logging
from .. import vc
from hcs_core.ctxp import CtxpException

log = logging.getLogger(__name__)


def _managed_obj_summary(obj):
    return {
        "type": obj["self"]["type"],
        "mor": obj["self"]["value"],
        "name": obj["name"],
    }


def _find_snapshot(input: dict):
    datacenter = input.get("datacenter")
    folder_path = input.get("folder_path")
    vm_name = input.get("vm_name")
    snapshot_name = input.get("snapshot_name")

    tree = vc.vm.get_snapshot_tree(datacenter_name=datacenter, folder_path=folder_path, vm_name=vm_name)

    def _is_target_node(n):
        if snapshot_name:
            return n.get("name") == snapshot_name
        else:
            return n.get("current")

    def _dfs(n):
        # print(n)
        if not n:
            return
        if _is_target_node(n):
            return n
        children = n.get("childSnapshotList")
        if not children:
            return
        for c in children:
            t = _dfs(c)
            if t:
                return t

    return _dfs({"childSnapshotList": tree})


def _find_snapshot2(input: dict):
    datacenter = input.get("datacenter")
    folder_path = input.get("folder_path")
    vm_name = input.get("vm_name")
    snapshot_name = input.get("snapshot_name")

    vm_info = vc.vm.info(datacenter_name=datacenter, folder_path=folder_path, vm_name=vm_name)

    def _is_target_node(n):
        if snapshot_name:
            return n.get("name") == snapshot_name
        else:
            return n["snapshot"]["value"] == vm_info["snapshot"]["currentSnapshot"]["value"]

    def _dfs(n):
        # print(n)
        if not n:
            return
        if _is_target_node(n):
            return n
        children = n.get("childSnapshotList")
        if not children:
            return
        for c in children:
            t = _dfs(c)
            if t:
                return t

    tree = {"snapshot": {"value": None}, "name": None, "childSnapshotList": vm_info["snapshot"]["rootSnapshotList"]}
    ret = _dfs(tree)

    if ret:
        ret["vmInstanceUuid"] = vm_info["config"]["instanceUuid"]
    return ret


def _find_datacenter(input: dict):
    name = input["name"]
    r = vc.datacenter.get(name)
    if not r:
        raise Exception("Datacenter not found: " + name)
    return _managed_obj_summary(r)


def _find_datastore(input: dict):
    name = input["name"]
    dc_name = input["datacenter"]
    r = vc.datastore.get(name, datacenter_name=dc_name)
    if not r:
        raise Exception("Datastore not found: " + name)
    ret = _managed_obj_summary(r)
    ret["datastoreType"] = "VSAN"  # TODO
    return ret


def _find_network(input: dict):
    name = input["name"]
    datacenter_name = input["datacenter"]
    r = vc.network.get(name, datacenter_name)
    if not r:
        raise Exception("Network not found: " + name)
    return _managed_obj_summary(r)


_fn_map = {
    "snapshot": _find_snapshot2,
    "datacenter": _find_datacenter,
    "datastore": _find_datastore,
    "network": _find_network,
}


def process(data: dict, state: dict) -> dict:
    ret = {}
    for k in data:
        type = data[k]["type"]
        input = data[k]["data"]
        fn = _fn_map.get(type)
        if not fn:
            raise Exception(f"Unknown type: {type}")
        ret[k] = fn(input)
    return ret


def destroy(data: dict, state: dict, force: bool):
    return


def eta(action: str, data: dict, state: dict):
    return "1m"



import subprocess
import json
import os

_env = {
    "GOVC_URL": None,
    "GOVC_USERNAME": None,
    "GOVC_PASSWORD": None,
    "GOVC_TLS_CA_CERTS": None,
    "GOVC_TLS_KNOWN_HOSTS": None,
    "GOVC_TLS_HANDSHAKE_TIMEOUT": None,
    "GOVC_INSECURE": "1",
    "GOVC_DATACENTER": None,
    "GOVC_DATASTORE": None,
    "GOVC_NETWORK": None,
    "GOVC_RESOURCE_POOL": None,
    "GOVC_HOST": None,
    "GOVC_GUEST_LOGIN": None,
    "GOVC_VIM_NAMESPACE": None,
    "GOVC_VIM_VERSION": None,
    "GOVC_VI_JSON": None,
}
_effective_env = {}


def init(config: dict = None):
    ignored = ["THUMBPRINT"]

    effective_config = {}
    if config:
        for k in config:
            k = k.upper()
            if k in ignored:
                continue
            v = config[k]
            k2 = k if k.startswith("GOVC_") else "GOVC_" + k
            if k2 not in _env:
                raise Exception("Unknown config: " + k)
            effective_config[k2] = v

    effective_config = os.environ | effective_config

    global _effective_env
    _effective_env = _env | effective_config

    keys = list(_effective_env.keys())
    for k in keys:
        if _effective_env[k] is None:
            del _effective_env[k]

    required = ["GOVC_URL"]

    for k in required:
        if not _effective_env[k]:
            raise Exception("Missing required GOVC config: " + k)

    # print("ENV:", json.dumps(_effective_env, indent=4))


def run(cmd, raise_on_failure: bool = False) -> subprocess.CompletedProcess:
    if type(cmd) == str:
        cmd = "govc " + cmd
    elif isinstance(cmd, list):
        cmd.insert(0, "govc")
    else:
        raise Exception(f"Unknown cmd argument type: {type(cmd).__name__}")

    print("CMD:", cmd)
    p = subprocess.run(
        cmd, capture_output=True, shell=False, cwd=None, timeout=None, check=False, text=True, env=_effective_env
    )

    if p.returncode and raise_on_failure:
        raise Exception(f"Fail running command: '{cmd}'. Return={p.returncode}, STDOUT={p.stdout}, STDERR={p.stderr}")
    return p


def get(cmd: str, as_json: bool = True, raise_on_failure: bool = False) -> dict:
    p = run(cmd, raise_on_failure)
    if p.returncode:
        if raise_on_failure:
            raise Exception(
                f"Fail running command: {p.args}. Return={p.returncode}, STDOUT={p.stdout}, STDERR={p.stderr}"
            )

    output = p.stdout
    # this looks like a govc inconsistency bug. E.g. govc snapshot.tree has output in stderr.
    if not output and p.returncode == 0 and p.stderr:
        output = p.stderr

    if output:
        if as_json:
            return json.loads(output)
        return p.stdout



from . import govc


def get(name: str, datacenter_name: str) -> dict:
    ret = govc.get(f"ls -json /{datacenter_name}/network/{name}", raise_on_failure=True)
    if ret:
        r = ret["elements"][0]["Object"]
        r["mor"] = r["self"]["value"]
        return r
